############################################################################
#                                                                          #
# A company is considering the opening of lockboxes in several cities to   #
# reduce the 'float' (lost interest) due to mail delay. In what cities     #
# should lockboxes be opened to minimize lost interst and operating cost?  #
# Each area can send payments to only one city.                            #
#                                                                          #
############################################################################

set AREAS;
set CITIES;

##############
# Parameters #
##############

param INTEREST;                     # Interest rate
param PAYMENTS {AREAS};             # Daily payments from areas
param OPERATING_COST {CITIES};      # Cost to operate lockbox per year 
param CLEARING_DAYS {AREAS,CITIES}; # Days from mailing to clearing

#############
# Variables #
#############

var ASSIGNMENTS {AREAS,CITIES} binary;   # Decision variables for areas
var LOCKBOXES {CITIES} binary;           # Lockbox decisions for cities
var LOST_INTEREST {AREAS,CITIES};        # Float for area and city

#########
# Model #
#########

minimize TOTAL_COST:
   sum {j in CITIES} OPERATING_COST[j]*LOCKBOXES[j] +
   sum {i in AREAS,j in CITIES} LOST_INTEREST[i,j];
subject to
   Calculate_lost_interest {i in AREAS, j in CITIES}: 
      LOST_INTEREST[i,j] = PAYMENTS[i]*CLEARING_DAYS[i,j]*ASSIGNMENTS[i,j]*
                           INTEREST;
   No_assignment_to_city_without_lockbox {i in AREAS,j in CITIES}:
      ASSIGNMENTS[i,j] <= LOCKBOXES[j];
   Only_one_lockbox_per_area {i in AREAS}:
      sum {j in CITIES} ASSIGNMENTS[i,j] = 1;
   

