#########################################################################
#                                                                       #
# An investor wants to put together a  portfolio consisting of a        #
# maximum of 5 stocks. What is the best combination of stocks to get    #
# the optimum return with a given variance (risk) ?                     #
#                                                                       #
#########################################################################

set STOCKS;
set PERIODS;

##############
# Parameters #
##############

param HISTDATA {PERIODS,STOCKS};  # Historical return of the stocks
param RISK;                       # The percentage risk the investor accepts

#############
# Variables #
#############

var PORTFOLIO_PART {STOCKS} >=0;  # The percentages of the portfolio
var RETURN {STOCKS};              # Expected return of each stock 
var VARIANCE {STOCKS,STOCKS};     # Variances and covariances of the stocks

#########
# Model #
#########

maximize PORTFOLIO_RETURN:
   sum {i in STOCKS} PORTFOLIO_PART[i]*RETURN[i];
subject to
   Complete_Portfolio:
      sum {i in STOCKS} PORTFOLIO_PART[i] = 1; 
   Calculate_return {i in STOCKS}:
      RETURN[i] = sum {g in PERIODS} HISTDATA[g,i] / card(PERIODS); 
   Calculate_variance {i in STOCKS,j in STOCKS}:
      VARIANCE[i,j] = 
        if i=j then
           sum {g in PERIODS} HISTDATA[g,i]^2/card(PERIODS) - RETURN[i]^2
        else
           sum {g in PERIODS} (HISTDATA[g,i]-RETURN[i])*(HISTDATA[g,j]-RETURN[j])
                          /card(PERIODS);
   The_Variance:   
      sum {i in STOCKS} PORTFOLIO_PART[i]*sum {j in STOCKS} PORTFOLIO_PART[j] 
          * VARIANCE[i,j] <= RISK;

