#########################################################################
#                                                                       #
# An investor wants to put together a  portfolio consisting of a        #
# maximum of 5 stocks. What is the best combination of stocks to get    #
# the optimum return with a given variance (risk) ?                     #
#                                                                       #
#########################################################################

set STOCKS;
set PERIODS;

##############
# Parameters #
##############

param HISTDATA {PERIODS,STOCKS};  # Historical return of the stocks
param MARKET_RETURN {PERIODS};    # Market return over the periods
param RISK;                       # The percentage risk the investor accepts

#############
# Variables #
#############

var PORTFOLIO_PART {STOCKS} >=0;  # The percentages of the portfolio
var ALPHA {STOCKS};               # Alphas of the Sharpe equations
var BETA {STOCKS};                # Betas of the Sharpe equations
var RESI {STOCKS};                # The residual error in the Sharp model

#########
# Model #
#########

maximize PORTFOLIO_RETURN:
   sum {i in STOCKS} (PORTFOLIO_PART[i]*ALPHA[i]+PORTFOLIO_PART[i]*BETA[i]
                      *sum {j in PERIODS} MARKET_RETURN[j]/card(PERIODS));
subject to
   Complete_Portfolio:
      sum {i in STOCKS} PORTFOLIO_PART[i] = 1; 
   Calculate_betas {i in STOCKS}:
      BETA[i] = ((card(PERIODS)*sum {g in PERIODS} HISTDATA[g,i]*MARKET_RETURN[g]) 
                - (sum {g in PERIODS} HISTDATA[g,i])*sum {g in PERIODS} MARKET_RETURN[g])
                / (card(PERIODS)* sum {g in PERIODS} MARKET_RETURN[g]^2-
                  (sum {g in PERIODS} MARKET_RETURN[g])^2);
   Calculate_alphas {i in STOCKS}:
      ALPHA[i] =((sum {g in PERIODS} HISTDATA[g,i])*sum {g in PERIODS} MARKET_RETURN[g]^2 -
                (sum {g in PERIODS} MARKET_RETURN[g])*sum {g in PERIODS} MARKET_RETURN[g]*HISTDATA[g,i])
                / (card(PERIODS)* sum {g in PERIODS} MARKET_RETURN[g]^2-
                  (sum {g in PERIODS} MARKET_RETURN[g])^2);
   Calculate_resis {i in STOCKS}:                
      RESI[i] = (card(PERIODS)*(sum {g in PERIODS} HISTDATA[g,i]^2)-((sum {g in PERIODS} HISTDATA[g,i])^2)
                -((card(PERIODS)*(sum {g in PERIODS} HISTDATA[g,i]*MARKET_RETURN[g])-
                  (sum {g in PERIODS} MARKET_RETURN[g])*(sum {g in PERIODS} HISTDATA[g,i]))^2) 
                  /(card(PERIODS)*(sum {g in PERIODS} MARKET_RETURN[g]^2)-
                   (sum {g in PERIODS} MARKET_RETURN[g])^2))/
                (card(PERIODS)*(card(PERIODS)-2));
   The_Variance:   
      (sum {i in STOCKS} BETA[i]*PORTFOLIO_PART[i])^2
      *(sum {g in PERIODS} MARKET_RETURN[g]^2/card(PERIODS) - (sum {g in PERIODS} MARKET_RETURN[g]/card(PERIODS))^2)
      + sum {i in STOCKS} PORTFOLIO_PART[i]^2*RESI[i]  <= RISK;

