###########################################################################
#                                                                         #
# A company currently ships its product from 5 plants to 4 warehouses. It # 
# is considering closing one or more plants to reduce cost. What plant(s) #
# should the company close in order to minimize transportation and fixed  #
# costs?                                                                  #
#                                                                         #
###########################################################################

set WAREHOUSES;
set PLANTS;

##############
# Parameters #
##############

param DEMAND {WAREHOUSES};                 # Demand of the warehouses x1000
param CAPACITY {PLANTS};                   # Maximum capacity of plants x1000
param FIXED_COST {PLANTS};                 # Cost to operate plant
param SHIPPING_COST {WAREHOUSES,PLANTS};   # Shipping cost

#############
# Variables #
#############

var DECISIONS {PLANTS} binary;                # Decision to open/close plants
                                              # 0 = closed, 1 = open
var PRODUCTS_TO_SHIP {WAREHOUSES,PLANTS} >=0; # Number of products to ship

#########
# Model #
#########

minimize TOTAL_COST:
   sum {i in WAREHOUSES,j in PLANTS} PRODUCTS_TO_SHIP[i,j]*SHIPPING_COST[i,j]+
   sum {j in PLANTS} DECISIONS[j]*FIXED_COST[j];
subject to
   Capacity {j in PLANTS}:
   sum {i in WAREHOUSES} PRODUCTS_TO_SHIP[i,j] <= CAPACITY[j]*DECISIONS[j];
   Demand {i in WAREHOUSES}:
   sum {j in PLANTS} PRODUCTS_TO_SHIP[i,j] = DEMAND[i];

