###########################################################################
#                                                                         #
# Minimize the costs of shipping goods from factories to customers while  #
# not exceeding the supply available from each factory and meeting the    #
# demand of each customer.                                                #
#                                                                         #
###########################################################################

set FACTORIES;
set CUSTOMERS;

##############
# Parameters #
##############

param FACTORY_CAPACITY {FACTORIES};        # Capacity of factories
param DEMAND {CUSTOMERS};                  # Demand of customers
param SHIPPING_COST {FACTORIES,CUSTOMERS}; # Shipping cost

#############
# Variables #
#############

var PRODUCTS_SHIPPED {FACTORIES,CUSTOMERS} >=0;        # Products shippped

#########
# Model #
#########

minimize TOTAL_COST:
   sum {i in FACTORIES, j in CUSTOMERS} 
       SHIPPING_COST[i,j]*PRODUCTS_SHIPPED[i,j];
subject to
  Demand {j in CUSTOMERS}:
     sum {i in FACTORIES} PRODUCTS_SHIPPED[i,j] = DEMAND[j];
  Factory_capacity {i in FACTORIES}:
     sum {j in CUSTOMERS} PRODUCTS_SHIPPED[i,j] <= FACTORY_CAPACITY[i];


    

