############################################################################
#                                                                          #
# Minimize the cost of operating different types of machines while meeting #
# demand. Each machine has a different cost and capacity. There are only a #
# limited number of machines available of each type.                       #
#                                                                          #
############################################################################

set MACHINES;

##############
# Parameters #
##############

param INITIAL_COST {MACHINES};               # Cost to start up machines
param ADDITIONAL_COST {MACHINES};            # Extra cost per product
param MAXIMUM_PRODUCTS {MACHINES};           # Maximum products per day
param AVAILABLE_MACHINES {MACHINES};         # Available machines
param DEMAND;                                # Required number of products

#############
# Variables #
#############

var MACHINES_TO_USE {MACHINES} integer;      # Number of machines to use
var PRODUCTS_TO_MAKE {MACHINES};             # Number of products to make

#########
# Model #
#########

minimize TOTAL_COST:
   sum {i in MACHINES} (MACHINES_TO_USE[i]*INITIAL_COST[i]+
                        PRODUCTS_TO_MAKE[i]*ADDITIONAL_COST[i]);
subject to
   Available_machines {i in MACHINES}:
      MACHINES_TO_USE[i] <= AVAILABLE_MACHINES[i];
   Capacity {i in MACHINES}:
      PRODUCTS_TO_MAKE[i] <= MACHINES_TO_USE[i]*MAXIMUM_PRODUCTS[i];
   Demand:
      sum {i in MACHINES} PRODUCTS_TO_MAKE[i] = DEMAND;

