##########################################################################
#                                                                        #
# What chemical plants should be used and how much should they produce   #
# to meet a certain quality (amounts of copper and silver) and minimize  #
# cost? There are several plants with different quality, capacity and    #
# cost to operate. A minimum output per year is required.                #
#                                                                        #
##########################################################################

set PLANTS;

##############
# Parameters #
##############

param RELATIVE_COPPER_CONTENTS {PLANTS};        # Compared to required
param RELATIVE_SILVER_CONTENTS {PLANTS};        #        ,,
param MAXIMUM_PRODUCTION {PLANTS};              # Maximum output per year
param REQUIRED;                                 # Total required output
param COPPER_REQUIRED;                          # Required copper contents
param SILVER_REQUIRED;                          # Required silver contents

#############
# Variables #
#############

var AMOUNTS_TO_PRODUCE {PLANTS} >=0;            # Amount per plant

#########
# Model #
#########

subject to 
   Output:
      sum {i in PLANTS} AMOUNTS_TO_PRODUCE[i] = REQUIRED;
   Maximum {i in PLANTS}:
      AMOUNTS_TO_PRODUCE[i] <= MAXIMUM_PRODUCTION[i];
   Copper:
      sum {i in PLANTS} AMOUNTS_TO_PRODUCE[i]*RELATIVE_COPPER_CONTENTS[i]
                        = REQUIRED*COPPER_REQUIRED;
   Silver:
      sum {i in PLANTS} AMOUNTS_TO_PRODUCE[i]*RELATIVE_SILVER_CONTENTS[i]
                        = REQUIRED*SILVER_REQUIRED;

