##########################################################################
#                                                                        #
# A large software company with separate buildings in different states,  #
# has offers from different floppy disk manufacturers to supply their    # 
# monthly need of diskettes. To whom should the contracts be awarded to  #
# minimize cost?                                                         #
# NOTE: Manufacturer 1 is only interested in contracts of certain size.  #
##########################################################################

set MANUFACTURERS ordered;
set BUILDINGS;

##############
# Parameters #
##############

param BIDS {MANUFACTURERS,BUILDINGS};            # Bids per 1000 diskettes
param CONTRACTS_AVAILABLE {MANUFACTURERS};       # Contracts available 
param CONTRACTS_REQUIRED {BUILDINGS};            # Contracts needed

#############
# Variables #
#############

var CONTRACTS_AWARDED {MANUFACTURERS,BUILDINGS} >=0; # Awarded contracts
# Manufacturer1 is only interested in contracts of 15 or more.
var DECISION binary;                                 # Use manufacturer1?

#########
# Model #
#########

minimize TOTAL_COST:
   sum {i in MANUFACTURERS,j in BUILDINGS} CONTRACTS_AWARDED[i,j]*BIDS[i,j];
subject to
   Manufacturer1 {i in MANUFACTURERS,j in BUILDINGS}:
      if i=first(MANUFACTURERS)
      then CONTRACTS_AWARDED[i,j] >= 15*DECISION;
   Availability {i in MANUFACTURERS}:
      sum {j in BUILDINGS} CONTRACTS_AWARDED[i,j] <= CONTRACTS_AVAILABLE[i];
   Demand {j in BUILDINGS}:
      sum {i in MANUFACTURERS} CONTRACTS_AWARDED[i,j] = CONTRACTS_REQUIRED[j];

